# Copyright (c) 2014, German Neuroinformatics Node (G-Node)
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted under the terms of the BSD License. See
# LICENSE file in the root of the Project.

VERSION = '1.4.1'
STATUS = 'Release'
RELEASE = '%s %s' % (VERSION, STATUS)
AUTHOR = 'Christian Kellner, Adrian Stoewer, Andrey Sobolev, Jan Grewe,\
Balint Morvai, Achilleas Koutsou'
COPYRIGHT = '2014, German Neuroinformatics Node, ' + AUTHOR
CONTACT = 'dev@g-node.org'
BRIEF = 'Python bindings for NIX'
HOMEPAGE = 'https://github.com/G-Node/nixpy'
